@@ -4,15 +4,19 @@ Provides functions that query the Korean Learners' Dictionary API.
 
 from . import scraper
 from .request import set_key
-from .response import set_default
 from .main import advanced_search, search, view
 from .types import (
     Classification,
+    DefinitionResponse,
+    ErrorResponse,
+    ExampleResponse,
+    IdiomProverbResponse,
     KRDictException,
-    MeaningCategory,
+    SemanticCategory,
     MultimediaType,
     OriginType,
     PartOfSpeech,
+    ResponseType,
     SearchMethod,
     SearchTarget,
     SearchType,
@@ -20,22 +24,28 @@ from .types import (
     SubjectCategory,
     TargetLanguage,
     TranslationLanguage,
-    VocabularyLevel
+    ViewResponse,
+    VocabularyLevel,
+    WordResponse
 )
 
 __all__ = [
     'scraper',
     'advanced_search',
     'search',
-    'set_default',
     'set_key',
     'view',
     'Classification',
+    'DefinitionResponse',
+    'ErrorResponse',
+    'ExampleResponse',
+    'IdiomProverbResponse',
     'KRDictException',
-    'MeaningCategory',
+    'SemanticCategory',
     'MultimediaType',
     'OriginType',
     'PartOfSpeech',
+    'ResponseType',
     'SearchMethod',
     'SearchTarget',
     'SearchType',
@@ -43,5 +53,7 @@ __all__ = [
     'SubjectCategory',
     'TargetLanguage',
     'TranslationLanguage',
-    'VocabularyLevel'
+    'ViewResponse',
+    'VocabularyLevel',
+    'WordResponse'
 ]
